import config

class BeamQueue:

    def __init__(self, size):

        self.size = size
        self.min_score = config._NEGATIVE_INFINITY
        self.min_elt = None
        self.queue = {}
    
    def add(self, phrase_nodes, new_node, phrase_edges, new_edge, new_elt_score):
        
        # If the beam queue is not full, add the element to the queue
        if len( self.queue ) < self.size:

            # Add to queue
            new_elt = self.__add_to_queue(phrase_nodes, new_node, phrase_edges, new_edge, new_elt_score)

            # Update the min score and min element
            if new_elt_score >= self.min_score:
                
                self.min_score = new_elt_score
                self.min_elt = new_elt

        # If the beam is full, and the new element score is greater
        # than or equal to the minimum score in the beam
        elif new_elt_score >= self.min_score:

            # Remove min element from queue
            if self.queue.has_key( self.min_elt ):
                del self.queue[self.min_elt]

            # Add this element to the queue
            new_elt = self.__add_to_queue(phrase_nodes, new_node, phrase_edges, new_edge, new_elt_score)

            # Find and set the new min element
            t_min_elt = new_elt
            t_min_elt_score = new_elt_score
            
            for elt, elt_score in self.queue.items():
                if t_min_elt_score >= elt_score:
                    t_min_elt = elt
                    t_min_elt_score = elt_score

            self.min_elt = t_min_elt
            self.min_elt_score = t_min_elt_score

        # Make sure the size of the queue is correct
        assert( len(self.queue) <= self.size )

    # Internal method to add an element to the queue
    def __add_to_queue(self, phrase_nodes, new_node, phrase_edges, new_edge, score):

        phrase_nodes = phrase_nodes + tuple([ new_node ])
        if new_edge != None: phrase_edges = phrase_edges + tuple([ new_edge ])

        new_elt = tuple([phrase_nodes, phrase_edges])
        self.queue[ new_elt ] = score

        return new_elt

    def items(self):
        for elt, elt_score in self.queue.items():
            yield elt, elt_score

    def top_element(self):

        # Make sure the beam is not empty
        assert( len(self.queue) > 0 )

        # Return the highest scoring element in the beam
        max_elt = None
        max_elt_score = config._NEGATIVE_INFINITY
        
        for elt, elt_score in self.queue.items():
            if elt_score >= max_elt_score:
                max_elt = elt
                max_elt_score = elt_score

        return (max_elt, max_elt_score)

