import re
import porter_stemmer
import os
import random
import config
from xml.dom import minidom, Node

###### CONSTANTS #########

# FLAGS
_STEMMING = True
_STOP_WORD_FILTERING = True

dataset = ""

pstemmer = porter_stemmer.PorterStemmer()
word_splitter = re.compile('[\s\#\!\@\#\$\%\^\&\*\(\)\+\=\[\]\{\}\\\|\;\:\'\"\<\>\,\.\?\/\`\~\-]')
punc_splitter = re.compile('[\#\!\@\#\$\%\^\&\*\(\)\+\=\[\]\{\}\\\|\;\:\'\"\,\.\?\/\`\~]')
sentence_splitter = None

def load_stop_words(stopwords_file):
    f = open(stopwords_file, "r")
    stopwords = f.read().strip().split("\n")
    f.close()

    return stopwords

stop_words = set(load_stop_words(config._STOPWORDS_FILE))

def get_xml_doc(name):
    return minidom.parse(name)

def split_into_sentences(text):

    sentences = sentence_splitter.split(text)

    # Remove blank sentences
    return_sentences = []
    for sentence in sentences:
        if len(sentence) > 0:
            return_sentences.append(sentence)

    return return_sentences

def split_into_words(text):
   words = word_splitter.split(text)
   return_words = []
   
   for word in words:
       if len(word) > 0 and ((_STOP_WORD_FILTERING and not is_stop_word(word)) or (not _STOP_WORD_FILTERING)):
           if _STEMMING:
               word = stem_word(word)
           return_words.append(word)
           
   return return_words

''' Splits a words into parts by punctuation '''
def split_by_punctuation(text):
    punc_parts = punc_splitter.split(text)
    new_punc_parts = []

    # Remove blank parts
    for punc_part in punc_parts:
        if punc_part != "" and punc_part != " ":
            new_punc_parts.append(punc_part)

    return new_punc_parts
    
def stem_word(word):
    return pstemmer.stem(word, 0, len(word)-1)

''' Splits text into unigrams, does not include unigrams across punctuation boundaries'''
def split_into_unigrams(text):
    return split_into_ngrams(text, 1)

''' Splits text into bigrams, does not include bigrams across punctuation boundaries'''
def split_into_bigrams(text):
    return split_into_ngrams(text, 2)

''' Splits text into trigrams, does not include trigrams across punctuation boundaries'''
def split_into_trigrams(text):
    return split_into_ngrams(text, 3)

''' Splits text into ngrams, does not include ngrams across punctuation boundaries'''
def split_into_ngrams(text, ngram_length):
    punc_parts = split_by_punctuation(text)
    ngrams = []
    
    for punc_part in punc_parts:
        words = split_into_words(punc_part)

        for i in range(len(words)-(ngram_length-1)):
            ngram = ""
            for j in range(i, i+ngram_length):
                ngram += words[j]+" "
            ngrams.append(ngram.strip())

    return ngrams

def is_stop_word(word):
    return word in stop_words

def extract_words_from_noun_phrase(np_text):
    return_words = []

    pieces = np_text.replace(")", "").split(" ")
    for piece in pieces:
        if "(" in piece:
            continue
        else:
            for word in split_into_words(piece):
                return_words.append(word)
            
    return return_words

def convert_to_hashset(l):
    hs = {}
    for e in l:
        hs[e] = 1
    return hs

def score_dice(cooccurence_count, word1_count, word2_count):
    return 2.0*cooccurence_count/(word1_count+word2_count)

def score_mi(cooccurence_count, word1_count, word2_count):
    return 1.0*cooccurence_count/(word1_count*word2_count)

def sort_dict_by_values(d):
    """ Returns the keys of dictionary d sorted by their values """
    items=d.items()
    backitems=[ [v[1],v[0]] for v in items]
    backitems.sort()
    return [ backitems[i][1] for i in range(0,len(backitems))]

def parse_text(text):
    # Split into sentences
    sentences = text.split("\n")

    # Write sentences to parse file
    temp_parse_file = "parse_file_tmp"+str(random.randint(0, 10000000))    
    sentence_file = open(temp_parse_file, 'w')
    for sentence in sentences:
        sentence_file.write("<s>("+sentence+")</s>\n")
    sentence_file.close()
        
    # Parse
    parse_pipe = os.popen(config._PARSER_CHARNIAK  + " " + temp_parse_file)
    parse_output = parse_pipe.read()
    parse_pipe.close()

    return parse_output
    

