package entities;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import utilities.Utils;

public class SGram {
	
	private List<String> sgramParts = new ArrayList<String> (); 
	public static final String WILDCARD_STRING = "#WILDCARD#";
	
	public SGram(List<String> sgramParts) {
		
		System.out.println(sgramParts);
		
		// Copy S Gram Parts
		this.sgramParts = sgramParts;

	}
	
	public boolean sentenceHasSgram(List <String> sentence) {
		
		int sgramPartsIndex = 0;
		for(String word : sentence) {
			// If all the sgramParts have been found, break
			if(sgramPartsIndex >= sgramParts.size()) {
				break;
			}
			
			String sgramPart = sgramParts.get(sgramPartsIndex);
			// If the current word is a part of the sgram, go to next sgram word
			if(sgramPart.equals(this.WILDCARD_STRING) || sgramPart.equals(word)) {
				sgramPartsIndex++;
			} else {
				return false;
			}
		}
		
		return sgramPartsIndex == this.sgramParts.size();
	}
	
	public static List<SGram> loadSgrams(String boostexterShypPath) {
		
		try {
			BufferedReader input = null;
			input = new BufferedReader( new FileReader(new File(boostexterShypPath)));
			String line = null; 
			
			List<SGram> sgrams = new ArrayList<SGram>(); 
			
			
			while (( line = input.readLine()) != null){
				line = line.trim();
				
				if(line.contains("Text:SGRAM:")) {
					
					// Get sgram parts
					String [] sgramParts = line.split("Text:SGRAM:")[1].split("#");
					List <String> sgramPartsList = new ArrayList<String>();
					for(int i=0; i < sgramParts.length; i++) {
						if(sgramParts[i].equals("?")) {
							sgramPartsList.add(SGram.WILDCARD_STRING);
						} else {
							sgramPartsList.add(Utils.stemWord(sgramParts[i]));
						}
					}
					
					// Create sgram
					sgrams.add(new SGram(sgramPartsList));
					
				}
			}
			
			return sgrams;
			
		} catch(IOException e) {
			System.out.println("* ERROR: Could not load SGrams "+e.toString());
		}
		
		return null;
	}

}
