package entities;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.List;

public class Tagger {

	private List <Hashtable<String, String>> sectionsStemToTags = new ArrayList<Hashtable<String, String>>();
	public static String UNKNOWN_TAG = "UNK";
	public static String SECTION_START_TAG = "####";
	public Collection <String> allPOSTags = new HashSet <String>();
	
	public Tagger(String perSectionPOSTagsFile) {
		this.allPOSTags.add(this.UNKNOWN_TAG);
		
		try {
			BufferedReader input = null;
			input = new BufferedReader( new FileReader(new File(perSectionPOSTagsFile)));
			String line = null; 
			
			Hashtable <String, String> stemToTags = null; 
			
			int lineNumber = 0;
			while (( line = input.readLine()) != null){
				line = line.trim();
				lineNumber++;
				// Move on to next section
				if(line.startsWith(SECTION_START_TAG)) {
					// For the first section, add it on the next loop iteration
					if(lineNumber > 1) {
						sectionsStemToTags.add(stemToTags);
					}
					stemToTags = new Hashtable <String, String> ();
				} else {
					// 	Split line into stem and tag
					String [] line_parts = line.split(" ");
					if(line_parts[1].equals(",") || line_parts[1].equals(".")) {
						line_parts[1] = this.UNKNOWN_TAG; 
					}
					stemToTags.put(line_parts[0], line_parts[1]);
					this.allPOSTags.add(line_parts[1]);
				}
			}
			// Add the last section
			sectionsStemToTags.add(stemToTags);
			
		} catch(IOException e) {
			System.out.println("* ERROR: Could not load POS tags "+e.toString());
		}
		
	}
	
	public String get(int sectionNumber, String stemmedWord) {
		Hashtable<String, String> stemToTagMap = sectionsStemToTags.get(sectionNumber);
		if(stemToTagMap.containsKey(stemmedWord)) {
			return stemToTagMap.get(stemmedWord);
		} else {
			return UNKNOWN_TAG;
		}
	}
	
	public Collection <String> getAllPOSTags() {
		return this.allPOSTags;
	}

}
