import re
import sys
import bigrams
import string
import pickle
import config
from xml.dom import minidom, Node

###### CONSTANTS #########

# DATA SET TYPES
_DATASET_CIVIL = "civil"
_DATASET_ALGORITHMS = "algorithms"
_DATASET_PHYSICS = "physics"
_DATASET_REUTERS = "reuters"

# DATA SET TYPE TO SECTION TAGNAME DICTIONARY
_dataset_to_section_tagname = {}
_dataset_to_section_tagname[_DATASET_CIVIL] = "part"
_dataset_to_section_tagname[_DATASET_ALGORITHMS] = "section"
_dataset_to_section_tagname[_DATASET_PHYSICS] = "section"
_dataset_to_section_tagname[_DATASET_REUTERS] = "newsitem" 

_MAX_CONSECUTIVE_STOP_WORDS = 2
_PICKLE_PATH = config._PICKLE_PATH

dataset = ""
word_splitter = re.compile('[\s\!\@\#\$\%\^\&\*\(\)\-\+\=\[\]\{\}\\\|\;\:\'\"\<\>\,\.\?\/\`\~]')

def get_section_titles_and_text(data_file, dataset_type, preserve_casing = False):
    print "** Reading text..."

    section_titles = []
    section_text = []

    xmldoc = bigrams.get_xml_doc(data_file)
    elements = xmldoc.getElementsByTagName(_dataset_to_section_tagname[dataset_type])

    for sec_num in range(len(elements)):
        element = elements[sec_num]

        title = ""
        text = ""
        
        # Get title and text
        if dataset_type == _DATASET_CIVIL:
            for cNode in element.childNodes:
                if cNode.nodeType == Node.ELEMENT_NODE:
                    if cNode.tagName == "title":
                        title = cNode.childNodes[0].nodeValue
                    elif cNode.tagName == "text":
                        if len(cNode.childNodes) > 0:
                            text = cNode.childNodes[0].nodeValue
                            text = re.sub('<.*?>', '', text).strip()
                            
        elif dataset_type == _DATASET_PHYSICS or dataset_type == _DATASET_ALGORITHMS:
            
            title = ( len( element.childNodes[0].childNodes ) > 0 ) and element.childNodes[0].childNodes[0].nodeValue or ""
            text = ( len( element.childNodes[1].childNodes ) > 0 ) and element.childNodes[1].childNodes[0].nodeValue or ""

        elif dataset_type == _DATASET_REUTERS:
            for child_node in element.childNodes:
                if child_node.nodeType == Node.ELEMENT_NODE:
                    if child_node.tagName == "headline":
                        title = child_node.childNodes[0].nodeValue
                    elif child_node.tagName == "text":
                        for grand_child_node in child_node.childNodes:
                            if grand_child_node.nodeType == Node.ELEMENT_NODE and grand_child_node.tagName == "p":
                                text += ((text!="") and "#SENTENCE#" or "")+grand_child_node.childNodes[0].nodeValue

        text = text.replace("#FIGURE#", "")

        if not preserve_casing:
            text = text.lower()

        section_titles.append(title)
        section_text.append(text)
        
    assert(len(section_titles) == len(section_text))

    return tuple([section_titles, section_text])

def get_section_titles(data_file, dataset_type):   
    (section_titles, section_text) = get_section_titles_and_text(data_file, dataset_type)
    section_titles_dict = {}

    for i, section_title in enumerate(section_titles):
        section_titles_dict[str(i)] = section_title

    return section_titles_dict

def get_per_section_stemming_probs(data_file, dataset_type):
    section_stem_probs = {}
    filename = _PICKLE_PATH+'section_stemprobs_'+dataset_type+'.pickle'
    try:
        # Try loading the pickle first
        stemfile = open(filename, 'r')
        section_stem_probs = pickle.load(stemfile)
        stemfile.close()
    except:
        # If not, recompute and dump
        stemfile = open(filename, 'w')
        section_stem_probs = _get_per_section_stemming_probs(data_file, dataset_type)
        pickle.dump(section_stem_probs, stemfile)
        stemfile.close()

    return section_stem_probs

def _get_per_section_stemming_probs(data_file, dataset_type):
    (section_titles, section_texts) = get_section_titles_and_text(data_file, dataset_type)

    sections = {}
    for i, text in enumerate(section_texts):
        sections[str(i)] = get_stemming_probs_from_text(text)
        
    return sections

def get_stemming_probs_from_text(text):
    stem_probs = {}

    # Get unstemmed words from this section
    bigrams._STOP_WORD_FILTERING = True
    bigrams._STEMMING = False
    unstemmed_words = bigrams.split_into_words(text)

    # Get stemmed words from this section
    bigrams._STOP_WORD_FILTERING = True
    bigrams._STEMMING = True
    stemmed_words = bigrams.split_into_words(text)

    # Check that each stemmed word has a corresponding unstemmed word
    assert len(stemmed_words) == len(unstemmed_words)

    # Update the unstemmed to stemmed count
    for word_index in range(len(unstemmed_words)):
        unstemmed_word = unstemmed_words[word_index]
        stemmed_word = stemmed_words[word_index]

        # Check if the stemmed word => unstemmed words dictionary exists
        if not stem_probs.has_key(stemmed_word):
            stem_probs[stemmed_word] = {}

        if not stem_probs[stemmed_word].has_key(unstemmed_word):
            stem_probs[stemmed_word][unstemmed_word] = 0
        stem_probs[stemmed_word][unstemmed_word] += 1

    # Convert to conditional probabilities
    for stemmed_word in stem_probs.keys():
        unstemmed_word_sum = sum(stem_probs[stemmed_word].values())
        for unstemmed_word in stem_probs[stemmed_word].keys():
            stem_probs[stemmed_word][unstemmed_word] /= 1.0*unstemmed_word_sum

    return stem_probs

def get_stemming_probs(data_file, dataset_type):
    stem_probs = {}
    filename = _PICKLE_PATH+'stemprobs_'+dataset_type+'.pickle'

    try:
        # Try loading the pickle first
        stemfile = open(filename, 'r')
        stem_probs = pickle.load(stemfile)
        stemfile.close()
    except:
        # If not, recompute and dump
        stemfile = open(filename, 'w')
        stem_probs = _get_stemming_probs(data_file, dataset_type)
        pickle.dump(stem_probs, stemfile)
        stemfile.close()

    return stem_probs

def _get_stemming_probs(data_file, dataset_type):
    print "* Reading text..."
    (section_titles, section_text) = get_section_titles_and_text(data_file, dataset_type)

    print "* Calculating stemming probabilites..."
    stem_probs = {}
    for text in section_text:

        # Get unstemmed words from this section
        bigrams._STOP_WORD_FILTERING = True
        bigrams._STEMMING = False
        unstemmed_words = bigrams.split_into_words(text)

        # Get unstemmed words from this section
        bigrams._STOP_WORD_FILTERING = True
        bigrams._STEMMING = True
        stemmed_words = bigrams.split_into_words(text)

        # Check that each stemmed word has a corresponding unstemmed word
        assert len(stemmed_words) == len(unstemmed_words)

        # Update the unstemmed to stemmed count
        for word_index in range(len(unstemmed_words)):
            
            unstemmed_word = unstemmed_words[word_index]
            stemmed_word = stemmed_words[word_index]

            # Check if the stemmed word => unstemmed words dictionary exists
            if not stem_probs.has_key(stemmed_word):
                stem_probs[stemmed_word] = {}

            if not stem_probs[stemmed_word].has_key(unstemmed_word):
                stem_probs[stemmed_word][unstemmed_word] = 0
            stem_probs[stemmed_word][unstemmed_word] += 1

    # Convert to conditional probabilities
    for stemmed_word in stem_probs.keys():
        unstemmed_word_sum = sum(stem_probs[stemmed_word].values())
        for unstemmed_word in stem_probs[stemmed_word].keys():
            stem_probs[stemmed_word][unstemmed_word] /= 1.0*unstemmed_word_sum

    return stem_probs

def get_per_section_unigram_probs(data_file, dataset_type):
    unigram_probs = {}
    filename = _PICKLE_PATH+'stemprobs_per_section_unigramprobs_'+dataset_type+'.pickle'

    try:
        # Try loading the pickle first
        unigramfile = open(filename, 'r')
        unigram_probs = pickle.load(unigramfile)
        unigramfile.close()
    except:
        print "* Calculating per-section unigram probs..."

        # If not, recompute and dump
        unigram_probs = _get_per_section_unigram_probs(data_file, dataset_type)
        unigramfile = open(filename, 'w')
        pickle.dump(unigram_probs, unigramfile)
        unigramfile.close()

    return unigram_probs

def get_unigram_probs(data_file, dataset_type):
    unigram_probs = {}
    filename = _PICKLE_PATH+'unigramprobs_'+dataset_type+'.pickle'
    
    try:
        # Try loading the pickle first
        unigramfile = open(filename, 'r')
        unigram_probs = pickle.load(unigramfile)
        unigramfile.close()
    except:
        # If not, recompute and dump
        unigramfile = open(filename, 'w')
        unigram_probs = _get_unigram_probs(data_file, dataset_type)
        pickle.dump(unigram_probs, unigramfile)
        unigramfile.close()

    return unigram_probs

def _get_per_section_unigram_probs(data_file, dataset_type):
    (section_titles, section_text) = get_section_titles_and_text(data_file, dataset_type)

    per_section_unigram_probs = {}
    for sec_num, text in enumerate(section_text):
        sec_num = str(sec_num)
        
        print "Extracting per-section unigrams: "+str(sec_num)+" "+section_titles[int(sec_num)]

        # Get unigrams from this section
        bigrams._STOP_WORD_FILTERING = False
        bigrams._STEMMING = False
        unigram_strings = bigrams.split_into_words(text)

        section_unigram_probs = {}
        
        # Update word counts
        for unigram_string in unigram_strings:
            if not section_unigram_probs.has_key(unigram_string):
                section_unigram_probs[unigram_string] = 0
            section_unigram_probs[unigram_string] += 1


        # Convert to conditional probabilities
        unigrams_sum = sum(section_unigram_probs.values())
        for unigram_string in section_unigram_probs.keys():
            section_unigram_probs[unigram_string] /= 1.0*unigrams_sum

        per_section_unigram_probs[sec_num] = section_unigram_probs
        
    return per_section_unigram_probs

def _get_unigram_probs(data_file, dataset_type):
    (section_titles, section_text) = get_section_titles_and_text(data_file, dataset_type)

    unigram_probs = {}
    for sec_num, text in enumerate(section_text):
        sec_num = str(sec_num)

        print "Extracting unigrams: "+str(sec_num)+" "+section_titles[int(sec_num)]
        
        # Get unigrams from this section
        bigrams._STOP_WORD_FILTERING = False
        bigrams._STEMMING = False
        unigram_strings = bigrams.split_into_words(text)

        # Update word counts
        for unigram_string in unigram_strings:
            if not unigram_probs.has_key(unigram_string):
                unigram_probs[unigram_string] = 0
            unigram_probs[unigram_string] += 1

    # Convert to conditional probabilities
    unigrams_sum = sum(unigram_probs.values())
    for unigram_string in unigram_probs.keys():
        unigram_probs[unigram_string] /= 1.0*unigrams_sum

    return unigram_probs

def get_per_section_bigram_probs(data_file, dataset_type):
    bigram_probs = {}
    filename = _PICKLE_PATH+'per_section_bigramprobs_'+dataset_type+'.pickle'
    
    try:        
        # Try loading the pickle first
        bigramfile = open(filename, 'r')
        bigram_probs = pickle.load(bigramfile)
        bigramfile.close()
    except:
        print "* Calculating per-section bigram probabilities..."
        
        # If not, recompute and dump
        bigram_probs = _get_per_section_bigram_probs(data_file, dataset_type)
        
        bigramfile = open(filename, 'w')
        pickle.dump(bigram_probs, bigramfile)
        bigramfile.close()
        
    return bigram_probs

def _get_per_section_bigram_probs(data_file, dataset_type):
    (section_titles, section_text) = get_section_titles_and_text(data_file, dataset_type)

    per_section_bigram_probs = {}
    for sec_num, text in enumerate(section_text):
        sec_num = str(sec_num)

        print "Extracting per-section bigrams: "+sec_num+" "+section_titles[int(sec_num)]
        
        # Get bigrams from this section
        bigrams._STOP_WORD_FILTERING = False
        bigrams._STEMMING = False
        bigram_strings = bigrams.split_into_bigrams(text)

        # Update the unstemmed to stemmed count
        section_bigram_probs = {}
        for bigram_string in bigram_strings:
            [word_one, word_two] = bigram_string.split(" ")

            # Check if the stemmed word => unstemmed words dictionary exists
            if not section_bigram_probs.has_key(word_one):
                section_bigram_probs[word_one] = {}

            if not section_bigram_probs[word_one].has_key(word_two):
                section_bigram_probs[word_one][word_two] = 0
            section_bigram_probs[word_one][word_two] += 1

        # Convert to conditional probabilities
        for word_one in section_bigram_probs.keys():
            word_one_sum = sum(section_bigram_probs[word_one].values())
            for word_two in section_bigram_probs[word_one].keys():
                section_bigram_probs[word_one][word_two] /= 1.0*word_one_sum

        per_section_bigram_probs[sec_num] = section_bigram_probs
        
    return per_section_bigram_probs

def get_bigram_probs(data_file, dataset_type, stopwords_list = []):
    bigram_probs = {}
    filename = _PICKLE_PATH+'bigramprobs_'+dataset_type+'.pickle'

    bigrams._STOP_WORD_FILTERING = False
    bigrams._STEMMING = False
    try:        
        # Try loading the pickle first
        bigramfile = open(filename, 'r')
        bigram_probs = pickle.load(bigramfile)
        bigramfile.close()
    except:
        # If not, recompute and dump
        bigramfile = open(filename, 'w')
        bigram_probs = _get_bigram_probs_laplace(data_file, dataset_type, stopwords_list)
        pickle.dump(bigram_probs, bigramfile)
        bigramfile.close()
        
    return bigram_probs

def get_stemmed_bigram_probs(data_file, dataset_type):
    bigram_probs = {}

    filename = _PICKLE_PATH+'stemmed_bigramprobs_'+dataset_type+'.pickle'

    bigrams._STOP_WORD_FILTERING = True
    bigrams._STEMMING = True
    try:        
        # Try loading the pickle first
        bigramfile = open(filename, 'r')
        bigram_probs = pickle.load(bigramfile)
        bigramfile.close()
    except:
        print "* Calculating stemmed bigram probabilities..."

        # If not, recompute and dump
        bigram_probs = _get_bigram_probs_laplace(data_file, dataset_type)
        
        bigramfile = open(filename, 'w')        
        pickle.dump(bigram_probs, bigramfile)
        bigramfile.close()
        
    return bigram_probs

def _get_bigram_probs_laplace(data_file, dataset_type, stopwords_list = []):
    stopwords_set = set(stopwords_list)
    bg_counts = {}
    unigrams = set()

    (section_titles, section_text) = get_section_titles_and_text(data_file, dataset_type)

    for sec_num, text in enumerate(section_text):
        sec_num = str(sec_num)

        print "Extracting bigrams: "+sec_num+" "+section_titles[int(sec_num)]

        
        # Get bigrams from this section
        bigram_strings = bigrams.split_into_bigrams(text)

        # Update the unstemmed to stemmed count
        for bigram_string in bigram_strings:
            [word_one, word_two] = bigram_string.split(" ")

            # Update set of unigrams
            unigrams.add(word_one)
            unigrams.add(word_two)

            bg_counts[word_one] = bg_counts.get(word_one, {})
            bg_counts[word_one][word_two] =  bg_counts[word_one].get(word_two, 0)+1
            
        # If a stop words list has been specified,
        # extract all trigram pairs
        if len(stopwords_list) > 0:
            trigram_strings = bigrams.split_into_trigrams(text)
            
            for trigram_string in trigram_strings:
                [word_one, word_two, word_three] =  trigram_string.split(" ")
                
                # If the first two words in the trigam form are a stop word, increment the count
                first_two_words = word_one+" "+word_two
                if first_two_words in stopwords_set:

                    # Update unigrams count
                    unigrams.add(first_two_words)
                    unigrams.add(word_three)
                    
                    bg_counts[first_two_words] = bg_counts.get(first_two_words,{})
                    bg_counts[first_two_words][word_three] = bg_counts[first_two_words].get(word_three, 0)+1

    # Calculate conditional probabilities with la place smoothing
    bigram_probs = {}
    vocabulary_size = len(unigrams)
    for i, word_one in enumerate(unigrams):

        print "Calculating Conditional Probabilites: "+str(i)+" of "+str(vocabulary_size)+" "+word_one

        # Get total count of word one
        word_one_sum = bg_counts.has_key(word_one) and sum(bg_counts[word_one].values()) or 0
        bigram_probs[word_one] = {}
        
        for word_two in unigrams:
            bigram_probs[word_one][word_two] = 1.0 * (bg_counts.get(word_one, {}).get(word_two, 0) + 1) / (word_one_sum + vocabulary_size)
            
            # Check that this generated a true probability
            assert bigram_probs[word_one][word_two] <= 1.0
            assert bigram_probs[word_one][word_two] >= 0.0
            
    # Print the total probability mass
    total_prob_mass = reduce(lambda x, y: x + sum(bigram_probs[y].values()), bigram_probs.keys(), 0.0)
    print "Total Probability Mass: "+str(total_prob_mass)
    assert(total_prob_mass == 1.0)

    return bigram_probs
    

def get_ngrams(data_file, dataset_type, max_ngram_length):
    ngram_list = {}
    filename = _PICKLE_PATH+'ngramlist_'+dataset_type+'_'\
               +str(max_ngram_length)+'.pickle'
    
    try:
        # Try loading the pickle first
        ngramfile = open(filename, 'r')
        ngrams_list = pickle.load(ngramfile)
        ngramfile.close()
    except:
        print "* Extracting ngram list..."
        # If not, recompute and dump
        ngramfile = open(filename, 'w')
        ngrams_list = _get_ngrams(data_file, dataset_type, max_ngram_length)
        pickle.dump(ngrams_list, ngramfile)
        ngramfile.close()

    return ngram_list

def get_stemmed_section_word_scores(maxent_probs_file, maxent_words_file):
    section = {}

    # Load words
    f = open(maxent_words_file, 'r')
    for line in f.readlines():
        [word, section_number] = line.strip().split(" ")
        section[section_number] = section.get(section_number, [])
        section[section_number].append(word)
    f.close()

    # Load probabilites
    probs = []
    f = open(maxent_probs_file, 'r')
    for line in f.readlines():
        line_parts = line.split(" ")
        prob = float(line_parts[line_parts.index("+title")+1].strip())
        assert(prob <= 1.0)
        probs.append(prob)
    f.close()

    # Create hash map word => in title probability
    section_word_scores = {}
    i = 0
    test_section_numbers = section.keys()
    test_section_numbers.sort()
    for section_number in test_section_numbers:
        section_word_scores[section_number] = {}
        for word in section[section_number]:
            section_word_scores[section_number][word] = probs[i]
            i += 1

    return section_word_scores

''' Read datafile and return hashset with the keys of all ngrams upto
length max_ngram_length.  Sentence boundaries are considered.'''
def _get_ngrams(data_file, dataset_type, max_ngram_length):
    ngrams = {}

    (section_titles, section_text) = get_section_titles_and_text(data_file, dataset_type)

    for sec_num, text in enumerate(section_text):
        sec_num = str(sec_num)

        # Set the sentence splitter and split text into sentences
        if dataset_type == _DATASET_ALGORITHMS or dataset_type == _DATASET_REUTERS:
            bigrams.sentence_splitter = re.compile('#sentence#') # Algorithms or Reuters
        else:
            bigrams.sentence_splitter = re.compile('[\.|\?|\!|\;] ') # Physics or Civil

        sentences = bigrams.split_into_sentences(text)
        for sentence in sentences:
            for ngram_length in range(1, max_ngram_length):
                for ngram in bigrams.split_into_ngrams(sentence, ngram_length):
                    if not ngrams.has_key(ngram):
                        ngrams[ngram] = 0
                    ngrams[ngram] += 1
                    
    return ngrams

def bigram_cartesian(string_list):
    cartesian_list = []

    for string_one in string_list:
        for string_two in string_list:
            if string_one != string_two:
                cartesian_list.append(string_one+" "+string_two)

    return cartesian_list

def load_all_stopwords(stopwords_file, datafile, dataset_type):

    print "* Loading stopwords..."
    single_stopwords = bigrams.load_stop_words(stopwords_file)
    cartesian_stopwords = bigram_cartesian(single_stopwords)

    # Load all text
    [s_titles, s_text] = get_section_titles_and_text(datafile, dataset_type)
    all_text = '\n'.join(s_text)
    # Set the sentence splitter and split text into sentences
    if dataset_type == _DATASET_ALGORITHMS or dataset_type == _DATASET_REUTERS:
        bigrams.sentence_splitter = re.compile('#sentence#') # Algorithms or Reuters
    else:
        bigrams.sentence_splitter = re.compile('[\.|\?|\!|\;] ') # Physics or Civil
        
    sentences = bigrams.split_into_sentences(all_text)
    all_text = '\n'.join(sentences)
    
    # Only consider stopwords which actually occur in the text
    cartesian_stopwords_use = []
    for cart_stopword in cartesian_stopwords:
        if all_text.count(cart_stopword) >= config._MIN_STOPWORD_OCCURENCES:
            cartesian_stopwords_use.append(cart_stopword)
            
    all_stopwords = []
    all_stopwords.extend(single_stopwords)
    # Remove spaces in stopwords
    bigram_stopwords = map(lambda x : x.replace(" ", "_"), cartesian_stopwords_use)
    all_stopwords.extend(bigram_stopwords)
    print "* Loaded "+str(len(all_stopwords))+" stopwords."

    return (single_stopwords, bigram_stopwords, all_stopwords)

def get_train_section_titles(maxent_stub):

    # Declare variables
    train_section_numbers = []
    
    # Read the section titles file
    f = open(maxent_stub+'.sectiontitles', 'r')
    lines = f.readlines()
    f.close()
    
    for num, line in enumerate(lines):
        if line.startswith("#TRAIN"):
         train_section_numbers.append(str(num))

    return train_section_numbers

def load_binary_selection_scores(sections_titles_file, section_words_file):
    selection_scores = {}

    # Load list of stemmed words in this section
    f = open(section_words_file, 'r')
    section_words = f.readlines()
    f.close()

    # Initially set all words in each test section to have
    # a selection score of zero
    for line in section_words:
        [word, secnum] = line.strip().split(" ")
        if not selection_scores.has_key(secnum):
            selection_scores[secnum] = {}
        selection_scores[secnum][word] = 0.0

    # Next load each section's title and set those words to have a
    # score of one only
    f = open(sections_titles_file, 'r')
    section_titles = f.readlines()
    f.close()

    # Loop through each section title
    for secnum, line in enumerate(section_titles):

        [train_or_test, title] = line.split(" ", 1)
        title = title.strip()
        train_or_test = train_or_test.strip()
        
        # If this is a not a test section skip it
        if train_or_test != "#TEST":
            continue

        # For each title word, set the selection score for the section
        # to 1
        title_words = title.split(" ")
        for title_word in title_words:

            if not selection_scores.has_key(str(secnum)):
                continue
            
            if selection_scores[str(secnum)].has_key(title_word):
                selection_scores[str(secnum)][title_word] = 1.0
        
    return selection_scores

def get_test_section_numbers(maxent_stub):
    return sorted(get_stemmed_section_word_scores(maxent_stub+".maxent_out", maxent_stub+".testwords").keys())
