package main;

import java.util.List;

import readers.AlgorithmsReader;
import readers.CivilReader;
import readers.PhysicsReader;
import readers.ReutersReader;
import entities.SGram;
import entities.Section;
import entities.Tagger;


public class Main {

	public static void main(String[] args) {

		List<Section> s;
		
		if(args.length < 2) {
			System.out.println("No dataset or test portion specified");
			 System.exit(1);
		}
		
		boolean includeHierarchicalFeatures = true;
		boolean includeLexicalFeatures = true;
		
		if(args[0].equals("physics")) {
			s = new PhysicsReader(true).readFile("/afs/csail.mit.edu/group/rbg/corpora/Physics/physics_rnd_baseline.xml");
			includeHierarchicalFeatures = true;
			includeLexicalFeatures = false;
		} else if(args[0].equals("algorithms")) {
			s = new AlgorithmsReader(true).readFile(args[1]);
			includeHierarchicalFeatures  = true;
			includeLexicalFeatures = false;
		} else if(args[0].equals("civil")) {
		    s = new CivilReader(true).readFile("/afs/csail.mit.edu/u/p/pawand/public_html/datasets/civil.xml");
		    includeHierarchicalFeatures = true;
		    includeLexicalFeatures = false;
		} else if(args[0].equals("reuters")) {
			s = new ReutersReader(true).readFile("/afs/csail.mit.edu/u/p/pawand/public_html/datasets/reuters.xml");
			includeHierarchicalFeatures = false;
			includeLexicalFeatures = true;
		} else {
			throw new RuntimeException("Invalid dataset specified");
		}
		
		
		Tagger tagger = HierarchicalFeatureVectorBuilder.FEATURE_POS_TAG?new Tagger("/afs/csail.mit.edu/u/p/pawand/dev/title_classification/graph_based/terminal_pickles/"+args[0]+"_selection_stem_to_tag.text"):null;
		List <SGram> sgrams = HierarchicalFeatureVectorBuilder.FEATURE_SGRAMS?SGram.loadSgrams("/afs/csail.mit.edu/u/p/pawand/dev/title_classification/lexical_feature_extractor/btstub_"+args[0]+".shyp"):null;
		HierarchicalFeatureVectorBuilder.BuildFeatureVectors(s, 
				"selection_out/title_class_"+args[0]+".data",
				"selection_out/title_class_"+args[0]+".test",
				"selection_out/title_class_"+args[0]+".testwords",
				"selection_out/title_class_"+args[0]+".sectiontitles",
				"selection_out/title_class_"+args[0]+".names",
				includeHierarchicalFeatures,
				includeLexicalFeatures,
				Float.parseFloat(args[2]),
				tagger,
				sgrams);
		
	}

}