class Node:

    ''' Node constructor '''
    def __init__(self, word, stemmed_word, number):
        self.stemmed_word = stemmed_word
        self.word = word
        self.out_edges = {}
        self.color = -1
        self.color_sets = {}
        self.number = number
        
    ''' Adds an edge from this node to another node with the specified edge weight '''
    def add_edge(self, edge):
        
        # Disallow self loops
        if edge.to_node == self:
            raise "Adding self-loop for node :"+self.word

        self.out_edges[edge.to_node] = edge

        # if not self.out_edges.has_key(edge.to_node):
        #      self.out_edges[edge.to_node] = []
        # self.out_edges[edge.to_node].append(edge)
        
    ''' Sets the color of this node '''
    def set_color(self, color):
        self.color = color

    ''' Adds a color set to this nodes collection of color sets '''
    def add_color_set(self, color_set):
        color_set_length = len(color_set.key)

        if not self.color_sets.has_key(color_set_length):
            self.color_sets[color_set_length] = {}
            
        self.color_sets[color_set_length][color_set.key] = color_set

    ''' Return all color sets with the specified color set key '''
    def get_color_set_by_colors(self, color_set_key):
        return self.color_sets[len(color_set_key)][color_set_key]

    ''' Checks if the node already has a color set with the given color set key '''
    def has_color_set_by_colors(self, color_set_key):
        return self.color_sets.get(len(color_set_key), {}).has_key(color_set_key)

    ''' Returns all colors sets of the given size '''
    def get_color_sets_by_size(self, size):
        return self.color_sets.get(size, {}).values()

    ''' Returns a string representation of this node '''
    def __str__(self):
        s =  "Node: Word: "+self.word+" Color:"+str(self.color)+"\n"
        s += "Colorsets: \n"
        for size, sized_color_sets in self.color_sets.items():
            for color_set in sized_color_sets.values():
                s += str(color_set)+"\n"
        s += "\n"
        return s
