package readers;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import entities.Section;


// This class can also be used to read 6.001 lectures
public class AlgorithmsFlatReader extends Reader {
	
	public AlgorithmsFlatReader(boolean enableStemming) {
		super();
		this.enableStemming = enableStemming;
	}
	
	public List<Section> readFile(String filePath) {
		System.out.print("Reading Algorithms Lecture..."); 
		
		List<Section> sections = new ArrayList<Section>();
		
		// Stream to read file
		FileInputStream fin;		
		
		try {
		    fin = new FileInputStream (filePath);
		    BufferedReader myInput = new BufferedReader(new InputStreamReader(fin));
		    
		    String line = "";
		    
		    // Done to avoid compiler warning
		    Section section = new Section("");
		    section = null;
		    
	        while ((line = myInput.readLine()) != null) {  
	        	// If the line is a section title
	        	if(line.startsWith("section ")) {
	        		
	        		// Add previous section to the list of sections
	        		if(section != null) {
	        			sections.add(section);
	        		}
	        		
	        		// Create next section
	        		String sectionTitle = cleanText(line.substring(8));
	        		section = new Section(sectionTitle.trim());

	        	// If the line is the body text
	        	} else if(section != null) {
	        		section.addSentence(cleanText(line.trim()));
	        	}
	        }

		    // Close our input stream
		    fin.close();		
		} catch (IOException e) {
			System.err.println ("Unable to read from file");
			System.exit(-1);
		}
		System.out.print("Done\n");
		
		return sections;
	}
	

}