package readers;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import utilities.Utils;
import entities.Section;

public class CivilReader extends Reader {

	public CivilReader(boolean enableStemming) {
		super();
		this.enableStemming = enableStemming;
	}

	public List<Section> readFile(String filePath) {
		System.out.print("Reading Civil Lecture...");
		try {
			
			List<Section> sections = new ArrayList<Section>();
			
	        DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
	        DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
	        Document doc = docBuilder.parse (new File(filePath));
	        
	        //	normalize text representation
            doc.getDocumentElement().normalize();
            NodeList sectionNodes = doc.getDocumentElement().getChildNodes();
            
            for(int sectionNum=0; sectionNum < sectionNodes.getLength(); sectionNum++){
            	Node sectionNode = sectionNodes.item(sectionNum);
        		if(sectionNode.getNodeName() == "part") {
        			sections.add(createSection(sectionNode));
        		}
            }
            
            System.out.print("Done\n");
            
            // Return a flat style list of all the in the section
            return flattenSections(sections);
            
        } catch (SAXParseException err) {
            
        	System.out.println ("** Parsing error" + ", line " 
                 + err.getLineNumber () + ", uri " + err.getSystemId ());
            System.out.println(" " + err.getMessage ());

        } catch (SAXException e) {
            
        	Exception x = e.getException ();
            ((x == null) ? e : x).printStackTrace ();

        } catch (Throwable t) {
        	
        	t.printStackTrace ();
        }
        return null;
	}
	
	public Section createSection(Node sectionNode) {
		
		String sectionTitle = "";
		String sectionBodyText = "";
		List <Section> sectionSubsections = new ArrayList<Section>();
		
		NodeList childNodes = sectionNode.getChildNodes();
		for(int i=0; i<childNodes.getLength(); i++) {
			Node childNode = childNodes.item(i);
			String childNodeName = childNode.getNodeName();
			
			if(childNodeName == "title") {
				sectionTitle = cleanText(childNode.getTextContent());
			} else if(childNodeName == "text") {
				sectionBodyText = cleanText(childNode.getTextContent());
			} else if(childNodeName == "part") {
				sectionSubsections.add(createSection(childNode));
			} else {
				//System.out.println("** Parsing error: Unrecognized Node Name "+childNode.getNodeName());
			}
		}
		
		// Create a new section
		Section section = new Section(sectionTitle);
		
		// Add sentences to section body
		for(String sentence : Utils.parseIntoSentences(sectionBodyText)){
			section.addSentence(sentence);
		}

		// Add subsections to section
		for(Section subsection : sectionSubsections) {
			section.addSubsection(subsection);
		}
		
		return section;
	}
}
