package readers;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import utilities.Utils;
import entities.Section;

public class PhysicsFlatReader extends Reader {

	public PhysicsFlatReader(boolean enableStemming) {
		super();
		this.enableStemming = enableStemming;
	}
	
	public List<Section> readFile(String filePath) {
		System.out.print("Reading Physics Flat Lecture...");
		try {
			
			List<Section> sections = new ArrayList<Section>();
			
	        DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
	        DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
	        Document doc = docBuilder.parse (new File(filePath));
	        
	        //	normalize text representation
            doc.getDocumentElement ().normalize();
            NodeList sectionNodes = doc.getDocumentElement().getChildNodes();
            
            for(int sectionNum=0; sectionNum < sectionNodes.getLength(); sectionNum++){
            	Node sectionNode = sectionNodes.item(sectionNum);
            	sections.addAll(createSection(sectionNode));
            }
            
            System.out.print("Done\n");
            return sections;
            
        } catch (SAXParseException err) {
            
        	System.out.println ("** Parsing error" + ", line " 
                 + err.getLineNumber () + ", uri " + err.getSystemId ());
            System.out.println(" " + err.getMessage ());

        } catch (SAXException e) {
            
        	Exception x = e.getException ();
            ((x == null) ? e : x).printStackTrace ();

        } catch (Throwable t) {
        	
        	t.printStackTrace ();
        }
        return null;
	}
	
	public List<Section> createSection(Node sectionNode) {
		List<Section> sections = new ArrayList<Section>();
		
		String sectionTitle = "";
		String sectionBodyText = "";
		
		NodeList childNodes = sectionNode.getChildNodes();
		for(int i=0; i<childNodes.getLength(); i++) {
			Node childNode = childNodes.item(i);
			String childNodeName = childNode.getNodeName();
			
			if(childNodeName == "title") {
				sectionTitle = cleanText(childNode.getTextContent());
			} else if(childNodeName == "body") {
				sectionBodyText = cleanText(childNode.getTextContent());
			} else if(childNodeName == "section") {
				sections.addAll(createSection(childNode));
			} else {
				System.out.println("** Parsing error: Unrecognized Node Name "+childNode.getNodeName());
			}
		}
		
		// Create a new section
		Section section = new Section(sectionTitle);
		
		// Add sentences to section body
		for(String sentence : Utils.parseIntoSentences(sectionBodyText)){
			section.addSentence(sentence);
		}
		
		// Add this section to beginning
		sections.add(0, section);
		
		return sections;
	}

}
