package readers;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import utilities.Utils;
import entities.Section;

public class Reader {

	Set<String> STOP_WORDS = new HashSet<String>();
	boolean enableStemming = true;
	private final String fillerwordsPath = "/afs/csail.mit.edu/u/p/pawand/dev/title_classification/qp/stopwords.txt";
	private final String stopwordsPath = "/afs/csail.mit.edu/u/p/pawand/dev/title_classification/qp/more_stopwords.txt";
	
	public Reader() {
		super();

		for(String word : this.loadStopwords(this.fillerwordsPath) ) {
			STOP_WORDS.add(word);
		}
		
		for(String word : this.loadStopwords(this.stopwordsPath) ) {
			STOP_WORDS.add(word);
		}

	}
	
	public Collection<String> loadStopwords(String stopwordsFilePath)  {
		Collection<String> stopwords = new ArrayList<String>();
		
		try {
			BufferedReader input = null;
			input = new BufferedReader( new FileReader(new File(stopwordsFilePath)) );
			String line = null; 
			
			while (( line = input.readLine()) != null){
				stopwords.add(line.trim().toLowerCase());
			}
		} catch(IOException e) {
			System.out.println("* ERROR: Could not load stop words "+e.toString());
		}
		return stopwords;
	}

	      

	public List<Section> readFile(String filePath) {
		// This should be implemented by the subclass
		return null;
	}
	
	String cleanText(String text) {
		// Remove stop words
		List<String> sentences = Utils.parseIntoSentences(text.toLowerCase());
		String newtext = "";
		for(String sentence : sentences) {
			List<String> words = Utils.parseIntoWords(sentence);
			for(String word : words) {
				if(!STOP_WORDS.contains(word) && !word.contains("&") && !word.contains("#") && !word.contains("_")) {
					if(enableStemming) {
						newtext +=  Utils.stemWord(word)+" ";
					} else {
						newtext +=  word+" ";
					}
				}
			}
		}
		return newtext;
	}
	
	public List<Section> flattenSections(List<Section> sections) {
		List<Section> sectionsFlat = new ArrayList<Section>();
		for(Section section : sections) {
			sectionsFlat.add(section);
			List<Section> subsections = section.getSubsections();
			if(subsections != null) {
				sectionsFlat.addAll(flattenSections(subsections));
			}
		}
		
		return sectionsFlat;
	}
}
