package readers;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import entities.Section;

public class ReutersReader extends Reader {
	
	public ReutersReader(boolean enableStemming) {
		super();
		this.enableStemming = enableStemming;
	}
	
	public List<Section> readFile(String filePath) {
		System.out.print("Reading Reuters...");
		try {
			
			List<Section> sections = new ArrayList<Section>();
			
	        DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
	        DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
	        Document doc = docBuilder.parse (new File(filePath));
	        
	        //	normalize text representation
            doc.getDocumentElement().normalize();
            NodeList sectionNodes = doc.getDocumentElement().getChildNodes();
            
            for(int sectionNum=0; sectionNum < sectionNodes.getLength(); sectionNum++){
            	Node sectionNode = sectionNodes.item(sectionNum);
        		if(sectionNode.getNodeName() == "newsitem") {
        			sections.add(createSection(sectionNode));
        		}
            }
            
            System.out.print("Done\n");
            
            // Return a flat style list of all the in the section
            return flattenSections(sections);
            
        } catch (SAXParseException err) {
            
        	System.out.println ("** Parsing error" + ", line " 
                 + err.getLineNumber () + ", uri " + err.getSystemId ());
            System.out.println(" " + err.getMessage ());

        } catch (SAXException e) {
            
        	Exception x = e.getException ();
            ((x == null) ? e : x).printStackTrace ();

        } catch (Throwable t) {
        	
        	t.printStackTrace ();
        }
        return null;
	}
	
	public Section createSection(Node newsitemNode) {
		
		String sectionTitle = "";
		List<String> sectionBody = new ArrayList<String>();
		List <Section> sectionSubsections = new ArrayList<Section>();
		
		NodeList childNodes = newsitemNode.getChildNodes();
		for(int i=0; i<childNodes.getLength(); i++) {
			Node childNode = childNodes.item(i);
			String childNodeName = childNode.getNodeName();
			
			if(childNodeName == "headline") {
				sectionTitle = cleanText(childNode.getTextContent());
				System.out.println(sectionTitle);
			} else if(childNodeName == "text") {
				NodeList sentenceNodes = childNode.getChildNodes();
				List<String> sentences = new ArrayList<String>();
				// Loop through each sentence node
				for(int j=0; j< sentenceNodes.getLength(); j++) {
					Node sentenceNode = sentenceNodes.item(j);
					// If this node is actually a setence
					if(sentenceNode.getNodeName() == "p") {
						sentences.add(sentenceNode.getTextContent());
					}
				}
				for(String sentence : sentences) {
					sectionBody.add(cleanText(sentence));
				}
			// Add subsections, this never occurs in the reuters corpus
			} else if(childNodeName == "newsitem") {
				sectionSubsections.add(createSection(childNode));
			}
		}
		
		// Create a new section
		Section section = new Section(sectionTitle);
		
		// Add sentences to section body
		for(String sentence : sectionBody){
			section.addSentence(sentence);
		}

		// Add subsections to section
		for(Section subsection : sectionSubsections) {
			section.addSubsection(subsection);
		}
		
		return section;
	}
}