#!/bin/bash

# Args: dataset, mode, job_id

##### CONFIGURABLE ########################
if [ $1 == "algorithms" ]; then
    dataset_name="algorithms"
    dataset_path="/afs/csail.mit.edu/u/p/pawand/public_html/datasets/algorithms.xml"
    percent_training=".70"
    beam_size="300"
    rand_iterations="20"
    boostexter_iterations=80
elif [ $1 == "reuters" ]; then
    dataset_name="reuters"
    dataset_path="/afs/csail.mit.edu/u/p/pawand/public_html/datasets/reuters.xml"
    percent_training=".90"
    beam_size="90"
    rand_iterations="5"
elif [ $1 == "physics" ]; then
    dataset_name="physics"
    dataset_path="/afs/csail.mit.edu/group/rbg/corpora/Physics/physics.xml"
    percent_training=".775"
    beam_size="90"
    rand_iterations="5"
    boostexter_iterations=90
elif [ $1 == "civil" ]; then
    dataset_name="civil"
    dataset_path="/afs/csail.mit.edu/u/p/pawand/public_html/datasets/civil.xml"
    percent_training=".90"
    beam_size="90"
    rand_iterations="5"
    boostexter_iterations=75
else
    echo "unknown datatype"
    exit
fi

BOOSTEXTER=/afs/csail.mit.edu/u/p/pawand/dev/boostexter/BoosTexter2_1/./boostexter

#################################

# Do not change
THRESHOLDER=maxent/threshold.pl
BT_TO_MAXENT_CONVERTER=maxent/conv_file_format.pl
MAXENT=maxent/me2

if [ $2 == "initialize" ]; then
    
    mkdir ${dataset_name}_graphs
    mkdir selection_out
    mkdir language_model_pickles
    mkdir srilm_files
    mkdir pickles
    mkdir terminal_pickles
    mkdir selection_out/me_model
    exit
fi

if [ $2 == "train" ]; then

    echo "** Generating feature vectors..."
    java -cp jar/selection.jar main.Main $dataset_name $dataset_path $percent_training

    echo "** Training with Boostexter..."
    $BOOSTEXTER -n $boostexter_iterations -S selection_out/title_class_${dataset_name}

    echo "** Testing with Boostexter..."
    $BOOSTEXTER -C -S selection_out/title_class_${dataset_name} -o < selection_out/title_class_${dataset_name}.test > selection_out/title_class_${dataset_name}.out
  
    echo "** Determining thresholds..."
    perl $THRESHOLDER -shyp selection_out/title_class_${dataset_name}.shyp selection_out/title_class_${dataset_name}.names selection_out/title_class_${dataset_name}.data selection_out/title_class_${dataset_name}.maxent_data_shyp
    perl $THRESHOLDER -shyp selection_out/title_class_${dataset_name}.shyp selection_out/title_class_${dataset_name}.names selection_out/title_class_${dataset_name}.test selection_out/title_class_${dataset_name}.maxent_test_shyp
   
    echo "** Converting to Max Entropy Feature vectors..."
    perl $BT_TO_MAXENT_CONVERTER selection_out/title_class_${dataset_name}.maxent_data_shyp.names -me2 < selection_out/title_class_${dataset_name}.maxent_data_shyp.data > selection_out/title_class_${dataset_name}.maxent_data | grep -v "dropped"
    perl $BT_TO_MAXENT_CONVERTER selection_out/title_class_${dataset_name}.maxent_test_shyp.names -me2 < selection_out/title_class_${dataset_name}.maxent_test_shyp.data > selection_out/title_class_${dataset_name}.maxent_test | grep -v "dropped"

    echo "** Training with Max Entropy..."
    $MAXENT -l -m selection_out/me_model < selection_out/title_class_${dataset_name}.maxent_data

    echo "** Testing with Max Entropy..."
    $MAXENT -m selection_out/me_model < selection_out/title_class_${dataset_name}.maxent_test > selection_out/title_class_${dataset_name}.maxent_out
  
    exit
fi

if [ $2 == "taglm" ]; then
    RE
    echo "* Tagging for language model and word to tag..."
    python2.4 search.py --mode=taglm --dataset=${dataset_name} --datafile=${dataset_path} --maxent_stub=selection_out/title_class_${dataset_name}
    exit
    
fi

# Build graphs
if [ $2 == "build" ]; then

    echo "Building graphs..."
    python2.4 search.py --mode=build --datafile=$dataset_path --dataset=${dataset_name} --maxent_stub=selection_out/title_class_${dataset_name} --graph_dir=${dataset_name}_graphs/ --lm_smoothing=true --tag_smoothing=true --job_id=$3
    
    exit
fi


if [ $2 == "rand" ]; then
    
    # TODO: Compute start and end sections and loop

    echo "* Running Randomized Decoder..."
    python2.4 search.py --mode=rand --dataset=${dataset_name} --datafile=${dataset_path} --maxent_stub=selection_out/title_class_algorithms --graph_dir${dataset_name}_graphs/ --sec_num=$i --job_id=$3 --ilp_outfile= --max_rand_iters=${rand_iterations} --maxent_stub=selection_out/title_class_${dataset_name}
    
    exit
fi

if [ $2 == "beam" ]; then
    
    # TODO: Compute start and end sections and loop

    echo "* Running Beam Search..."
    python2.4 search.py --mode=beam --dataset=${dataset_name} --datafile=${dataset_path} --maxent_stub=selection_out/title_class_${dataset_name} --graph_dir=${dataset_name}_graphs/ --beam_size=${beam_size} --sec_num=$i --job_id=$3 --maxent_stub=selection_out/title_class_${dataset_name}
    exit
fi

if [ $2 == "clean" ]; then
    
    echo "* Removing selection feature vectors..."
    rm selection_out/title_class_${dataset_name}.*

    echo "* Removing non-SRILM language model..."
    rm language_model_pickles/${dataset_name}.lang_model.pickle

    echo "* Removing SRILM bigram language model..."
    rm srilm_files/lm_arpa_${dataset_name}_2.lmfile

    echo "* Removing SRILM tagger language model..."
    rm srilm_files/lm_arpa_${dataset_name}_2_tags.lmfile

    echo "* Removing destemming pickle..."
    rm pickles/section_stemprobs_${dataset_name}.pickle
    rm pickles/stemprobs_${dataset_name}.pickle

    echo "* Removing per stem to tag (sometimes used for selection features)..."
    rm terminal_pickles/${dataset_name}_selection_stem_to_tag.text
    
    echo "* Removing per title tag language model..."
    rm terminal_pickles/${dataset_name}_tag_lm.pickle

    echo "* Removing word to tag pickle..."
    rm terminal_pickles/${dataset_name}_word_to_tag.pickle
    
    exit

fi