import use_psyco, lattice, utils, config, bigrams
import os, re

class SRILM:

    def __init__(self, dataset_type, dataset_file, cartesian_stopwords = []):

        self.unigram_scores = {}
        self.unigram_backoffs = {}
        self.bigram_scores = {}
        
        order = 2
        
        # Set the sentence splitter and split text into sentences
        if dataset_type == lattice._DATASET_ALGORITHMS or dataset_type == lattice._DATASET_REUTERS:
            bigrams.sentence_splitter = re.compile('#SENTENCE#') # Algorithms or Reuters
        else:
            bigrams.sentence_splitter = re.compile('[\.|\?|\!|\;] ') # Physics or Civil

        self.lm_file = config._SRILM_LM_FILE_PATH + "lm_arpa_" + dataset_type + "_" + str(order) + ".lmfile"

        # Check if language model already exists
        if not utils.fileexists(self.lm_file):
            
            # Otherwise build the language model
            # Generate language model text file
            [section_titles, section_text] = lattice.get_section_titles_and_text(dataset_file, dataset_type, False)
            all_text = '\n'.join(map(lambda text: '\n'.join(bigrams.split_into_sentences(text)), section_text))
            
            # Replace stop word bigrams with single word chars
            for stopword_bigram in cartesian_stopwords:
                all_text.replace(' '.join(stopword_bigram.split("_")), stopword_bigram)
                
            # Write text to file
            train_text_file = config._SRILM_LM_TEMP_FILE_PATH() + "train_file.text"
            f = open(train_text_file, 'w')
            f.write(all_text)
            f.close()
            
            # Clean punctuation
            lp_pipe = os.popen(config._TOKENIZER + " " + train_text_file)
            lp_output = lp_pipe.read().replace("-", " ")
            lp_pipe.close()

            f = open(train_text_file, 'w')
            f.write(lp_output)
            f.close()

            # Train language model with SRILM
            print "* Training...",
            lp_pipe = os.popen(config._SRILM_PATH + "ngram-count -order " + str(order) + " -text " + train_text_file + " -lm " + self.lm_file)
            lp_output = lp_pipe.read()
            lp_pipe.close()
            print "Done."

        self.__read_arpa_output(self.lm_file)
        
    def __read_arpa_output(self, arpa_file):

        f = open(arpa_file, 'r')
        arpa_file_contents = f.read()
        f.close()

        lines = arpa_file_contents.split("\n")

        level = 0
        for line in lines:

            line = line.strip()

            # Find start of unigrams
            if level == 0:

                if "\\1-grams:" in line:
                    level = 1
                    continue
                else:
                    continue

            # Read unigrams, find start of bigrams
            elif level == 1:

                if "\\2-grams:" in line:

                    level = 2
                    continue

                elif line == "":
                    continue
                else:

                    line_parts = line.split("\t")

                    if len(line_parts) == 2:
                        
                        [score, unigram] = line_parts
                        self.unigram_scores[unigram] = float(score)
                        
                    elif len(line_parts) == 3:

                        [score, unigram, backoff] = line_parts
                        self.unigram_backoffs[unigram] = float(backoff)
                        self.unigram_scores[unigram] = float(score)

                    else:

                        raise Exception("Unknown line split: "+line)
                        
            # Read bigrams
            elif level == 2:
            
                if line == "" or line == "\\end\\":
                    continue

                else:

                    [score, bigram] = line.split("\t")
                    [first_word, second_word] = bigram.split(" ")

                    self.bigram_scores[(first_word, second_word)] = float(score)
            else:
                
                raise Exception("Unknown level "+str(level))
                
    def get(self, first_word, second_word):

        return 10 ** ( self.bigram_scores.get((first_word, second_word), \
                                              self.unigram_scores.get(second_word, 0.0)+self.unigram_backoffs.get(first_word, 0.0)) )

