import use_psyco, lattice, config, tagger, utils
import os, string

class SRILM_tagger:

    def __init__(self, dataset, datafile, training_sections):

        order = 2
        
        lm_file = config._SRILM_LM_FILE_PATH + "lm_arpa_" + dataset + "_" + str(order) + "_tags.lmfile"
        
        self.unigram_scores = {}
        self.unigram_backoffs = {}
        self.bigram_scores = {}

        # Check if language model already exists
        if not utils.fileexists(lm_file):
            
            print "** Building title tag language model..."

            # Get all titles
            section_titles = lattice.get_section_titles(datafile, dataset)
            
            all_titles = []
            # For each training section
            for train_section in training_sections:
                title = section_titles[str(train_section)]
            
                # If the title does not end with a punctuation character, add it
                if len(title) > 0 and not title[-1] in string.punctuation:
                    title = title+"."

                    all_titles.append(tagger.clean_text(dataset, title))

            # Tag and update bigram and unigram counts
            tagged_titles = tagger.tag_text_mxpost(all_titles)

            # Write tags to file
            # Drop the trailing punctuation tag
            train_text_file = config._SRILM_LM_TEMP_FILE_PATH() + "tag_train_file.text"
            f = open(train_text_file, 'w')
            for (tags, words) in tagged_titles:
                f.write(' '.join(tags[:-1])+"\n")
            f.close()
                    
            # Train language model with SRILM
            print "* Training...",
            lp_pipe = os.popen(config._SRILM_PATH + "ngram-count -order " + str(order) + " -text " + train_text_file + " -lm " + lm_file)
            lp_pipe.close()
            print "Done."
        
        self.__read_arpa_output(lm_file)
        
    def __read_arpa_output(self, arpa_file):

        f = open(arpa_file, 'r')
        arpa_file_contents = f.read()
        f.close()

        # Replace "<s>" with "#START#" and Replace "</s>" with "#END#"
        arpa_file_contents = arpa_file_contents.replace("<s>", "#START#").replace("</s>", "#END#")
        
        lines = arpa_file_contents.split("\n")

        level = 0
        for line in lines:

            line = line.strip()

            # Find start of unigrams
            if level == 0:

                if "\\1-grams:" in line:
                    level = 1
                    continue
                else:
                    continue

            # Read unigrams, find start of bigrams
            elif level == 1:

                if "\\2-grams:" in line:

                    level = 2
                    continue

                elif line == "":
                    continue
                else:

                    line_parts = line.split("\t")

                    if len(line_parts) == 2:
                        
                        [score, unigram] = line_parts
                        self.unigram_scores[unigram] = float(score)
                        
                    elif len(line_parts) == 3:

                        [score, unigram, backoff] = line_parts
                        self.unigram_backoffs[unigram] = float(backoff)
                        self.unigram_scores[unigram] = float(score)

                    else:

                        raise Exception("Unknown line split: "+line)
                        
            # Read bigrams
            elif level == 2:
            
                if line == "" or line == "\\end\\":
                    continue

                else:
                    
                    [score, bigram] = line.split("\t")
                    [first_word, second_word] = bigram.split(" ")

                    self.bigram_scores[(first_word, second_word)] = float(score)
            else:
                
                raise Exception("Unknown level "+str(level))
                
    def get(self, first_word, second_word):
        return 10 ** ( self.bigram_scores.get((first_word, second_word), \
                                              self.unigram_scores.get(second_word, 0.0)+self.unigram_backoffs.get(first_word, 0.0)) )
