import use_psyco, lattice, config, bigrams, utils
import os, string, re

def clean_text(dataset, text):
    # Replace <num> with a dummy number
    # Replace <expression> with a dummy exression
    if dataset == lattice._DATASET_ALGORITHMS:
        return text.replace("<num>", "99999").replace("<expression>", "ABC").replace("line 99999", "ABCDEF").replace("<", "").replace(">", "")
    return text

# Tag every word in corpus and learn majority tag mapping
''' Tags the whole corpus text and builts a dictionary from each word to its most popular tag '''
def majority_tag_corpus(datafile, dataset):

    print "** Finding majority tag in corpus..."

    # Variable declaration
    word_to_tags = {}
    word_to_tag = {}
    
    # Get all text
    (section_titles, section_text) = lattice.get_section_titles_and_text(datafile, dataset, True)

    # Split into sentences
    sentences = []    
    # Set the sentence splitter and split text into sentences
    if dataset == lattice._DATASET_ALGORITHMS or dataset == lattice._DATASET_REUTERS:
        bigrams.sentence_splitter = re.compile('#SENTENCE#') # Algorithms or Reuters
    else:
        bigrams.sentence_splitter = re.compile('[\.|\?|\!|\;] ') # Physics or Civil
    for s_text in section_text:
        sentences.extend(bigrams.split_into_sentences(clean_text(dataset, s_text)))

    # Tag each sentence
    tagged_sentences = tag_text_mxpost(sentences, True)
    for sentence_num, (tags, words) in enumerate(tagged_sentences):

        # Update word to tag count
        for index, word in enumerate(words):

            # Get corresponding tag
            tag = tags[index]

            # Lower case the word
            word = word.lower()
            
            # Check if this word has been indexed before
            if not word_to_tags.has_key(word):
                word_to_tags[word] = {}

            # Update count
            word_to_tags[word][tag] = word_to_tags[word].get(tag, 0) + 1
        
    # Map the majority tag for each word
    for word in word_to_tags.keys():
        max_count = 0
        max_tag = None
        
        for (tag, count) in word_to_tags[word].items():

            if count >= max_count:
                max_tag = tag
                max_count = count

        word_to_tag[word] = max_tag

    return word_to_tag

# Tag every title in corpus and build a bigram language model
def build_title_tag_lm(datafile, dataset, training_sections):    

    print "** Building title tag language model..."

    # Get all titles
    section_titles = lattice.get_section_titles(datafile, dataset)

    tag_unigram_counts = {}
    tag_bigram_counts = {}

    all_titles = []
    # For each training section
    for train_section in training_sections:
        title = section_titles[train_section]

        # If the title does not end with a punctuation character, add it
        if len(title) > 0 and not title[-1] in string.punctuation:
            title = title+"."

        all_titles.append(clean_text(dataset, title))

    # Tag and update bigram and unigram counts
    tagged_titles = tag_text_mxpost(all_titles)

    for (tags, words) in tagged_titles:

        tags = tags[:-1]
        words = map(lambda word : word.lower(),  words[:-1])
        
        # Add start and end tag to tags
        tags.insert(0, "#START#")
        tags.append("#END#")

        # Update bigram counts
        for tag_index in range(len(tags)-1):
            tag_one = tags[tag_index]
            tag_two = tags[tag_index+1]
    
            tag_bigram_counts[(tag_one, tag_two)] = tag_bigram_counts.get((tag_one, tag_two), 0) + 1
    
        # Update unigram counts
        for tag in tags:
            tag_unigram_counts[tag] = tag_unigram_counts.get(tag, 0) + 1


    return (tag_bigram_counts, tag_unigram_counts)

# Now normalize the counts to get a probability
def normalize(counts_dict):

    total_count = sum(counts_dict.values())

    print counts_dict
    
    for tag in counts_dict.keys():
        counts_dict[tag] = 1.0 * counts_dict[tag] / total_count

def tag_text_mxpost(sentences, end_punctuation = False):

    # Skip if there are no sentences to tag
    if len(sentences) == 0: return []
    
    print "* Tagging "+str(len(sentences))+" sentences..."

    # Write sentences to file, one sentence per line
    f = open(config._TAGGER_TEMP_FILE, 'w')
    for sentence in sentences:
        # Add puncuation to the end of the sentence if needed
        if end_punctuation and not sentence[-1] in string.punctuation:
            sentence = sentence+"."
        f.write(sentence+"\n")
    f.close()

    # Tokenize sentences
    p = os.popen(config._TOKENIZER+' < '+config._TAGGER_TEMP_FILE)
    tokenized_text = p.read().replace("-", " ")
    p.close()

    # Write tokenized text to file
    f = open(config._TAGGER_TEMP_FILE, 'w')
    f.write(tokenized_text)
    f.close()

    # Tag tokenized text
    p = os.popen(config._MXPOST_TAGGER+' '+config._MXPOST_TAGGER_PROJECT+' < '+config._TAGGER_TEMP_FILE)
    tagged_output = p.read()
    p.close()

    # Write tagged output to file
    # Write tokenized text to file
    f = open(config._TAGGER_TEMP_FILE, 'w')
    f.write(tagged_output)
    f.close()
    
    # Return words and tags lists
    tagged_sentences = []
    for tagged_sentence in tagged_output.strip().split("\n"):

        #if tagged_sentence == "":
        #    continue
                
        sentence_tags = []
        sentence_words = []
        
        for word_tag in tagged_sentence.split(" "):
            if word_tag == "":
                continue
            
            (word, tag) = word_tag.rsplit("_", 1)

            sentence_tags.append(tag)
            sentence_words.append(word)
        
        assert(len(sentence_tags) == len(sentence_words))
        tagged_sentences.append((sentence_tags, sentence_words))

    assert(len(tagged_sentences) == len(sentences))
    return tagged_sentences


def majority_tag_per_section(dataset, datafile, section_numbers):

    print "** Finding majority tag per section..."

    # Get all text
    (section_titles, section_text) = lattice.get_section_titles_and_text(datafile, dataset, True)

    # Set the sentence splitter and split text into sentences
    if dataset == lattice._DATASET_ALGORITHMS or dataset == lattice._DATASET_REUTERS:
        bigrams.sentence_splitter = re.compile('#SENTENCE#') # Algorithms or Reuters
    else:
        bigrams.sentence_splitter = re.compile('[\.|\?|\!|\;] ') # Physics or Civil

    per_section_word_to_tag = {}
        
    # Loop through each section
    for section_number in section_numbers:

        print "* Tagging section "+str(section_number)+": "+section_titles[int(section_number)]+"..."
        
        # Variable declaration
        s_text = section_text[int(section_number)]
        word_to_tags = {}
    
        sentences = bigrams.split_into_sentences(clean_text(dataset, s_text))

        # Tag each sentence
        tagged_sentences = tag_text_mxpost(sentences, True)
        for sentence_num, (tags, words) in enumerate(tagged_sentences):

            # Update word to tag count
            for index, word in enumerate(words):

                # Lower case the word
                word = word.lower()
                
                # Get corresponding tag
                tag = tags[index]
                
                # Check if this word has been indexed before
                if not word_to_tags.has_key(word):
                    word_to_tags[word] = {}

                # Update count
                word_to_tags[word][tag] = word_to_tags[word].get(tag, 0) + 1
        
        # Map the majority tag for each word
        for word in word_to_tags.keys():
            max_count = 0
            max_tag = None
        
            for (tag, count) in word_to_tags[word].items():
                if count >= max_count and count >= config._MINIMUM_TAGS_PER_WORD_IN_SECTION:
                    max_tag = tag       
                    max_count = count
                    
            if max_tag != None:
                per_section_word_to_tag[(word, section_number)] = max_tag

    return per_section_word_to_tag

def majority_stem_tag_per_section(dataset, datafile):

    print "** Finding majority stem tag per section..."

    # Get all text
    (section_titles, section_text) = lattice.get_section_titles_and_text(datafile, dataset, True)

    # Set the sentence splitter and split text into sentences
    if dataset == lattice._DATASET_ALGORITHMS or dataset == lattice._DATASET_REUTERS:
        bigrams.sentence_splitter = re.compile('#SENTENCE#') # Algorithms or Reuters
    else:
        bigrams.sentence_splitter = re.compile('[\.|\?|\!|\;] ') # Physics or Civil

    per_section_stem_to_tag = []

    # Loop through each section
    for section_number, s_text in enumerate(section_text):

        s_text = section_text[int(section_number)]
        # Variable declaration
        stem_to_tags = {}
    
        sentences = bigrams.split_into_sentences(clean_text(dataset, s_text))

        # Tag each sentence
        tagged_sentences = tag_text_mxpost(sentences, True)
        for sentence_num, (tags, words) in enumerate(tagged_sentences):

            # Update stem to tag count
            for index, word in enumerate(words):
                
                # Get corresponding tag
                tag = tags[index]

                # Lower case the word
                word = word.lower()
                
                # Check if this stem has been indexed before
                stem = bigrams.stem_word(utils.strip_punc(word))

                # Skip if empty
                if(stem == ""):
                    continue
                
                if not stem_to_tags.has_key(stem):
                    stem_to_tags[stem] = {}

                # Update count
                stem_to_tags[stem][tag] = stem_to_tags[stem].get(tag, 0) + 1

        stem_to_tag_store = {}
        # Map the majority tag for each stem
        for stem in stem_to_tags.keys():
            max_count = 0
            max_tag = None
        
            for (tag, count) in stem_to_tags[stem].items():
                if count >= max_count:
                    max_tag = tag       

            stem_to_tag_store[stem] = max_tag
        per_section_stem_to_tag.append(stem_to_tag_store)

    return per_section_stem_to_tag

class WordToTag:
    def __init__(self, dataset, datafile, test_section_numbers):
        # Get majority tag over whole corpus
        self.majority_word_to_tag = majority_tag_corpus(datafile, dataset)

        # Get majority tag per sections
        self.majority_per_section = majority_tag_per_section(dataset, datafile, test_section_numbers)

    def get(self, word, section_number):
        return self.majority_per_section.get((word, section_number), self.majority_word_to_tag.get(word, ""))

class TitleTagLanguageModel:
    def __init__(self, datafile, dataset, train_sections):
        (self.bigram_counts, self.unigram_counts) = build_title_tag_lm(datafile, dataset, train_sections)
        self.total_unigrams = sum(self.unigram_counts.values())

    # Return bigram probability with LaPlace smoothing
    def get(self, tag_one, tag_two):
        if config._TITLE_TAG_LANGUAGE_MODEL_LAPLACE_SMOOTHING:
            return (1.0 * (self.bigram_counts.get((tag_one, tag_two), 0) + 1) / (self.unigram_counts.get(tag_one, 0) + self.total_unigrams))
        else:
            return self.bigram_counts.has_key((tag_one, tag_two)) and (1.0 * self.bigram_counts[(tag_one, tag_two)] / self.unigram_counts[tag_one]) or 0.0

