import lattice, utils, config
import sys, os, math

def get_terminal_word_probs(datafile, dataset, maxent_stub):

    order = 2

    # Try to load from pickle
    pickle_path = config._LANGUAGE_MODEL_PICKLE_PATH+dataset+"_terminal_word_probs.pickle"
    try:
        return utils.load_pickle(pickle_path)
    except:
        print "* Failed to load from pickle..."

    start_word_probs = {}
    end_word_probs = {}

    print "* Loading section words stemmed with scores..."
    non_binary_stemmed_section_word_scores = lattice.get_stemmed_section_word_scores(maxent_stub+".maxent_out", maxent_stub+".testwords")
    
    stemmed_word_scores = non_binary_stemmed_section_word_scores
    
    # Get test section numbers
    test_sections = sorted(stemmed_word_scores.keys())

    # Get destemming probabilities        
    print "* Loading destemming probabilities..."
    stem_probs = lattice.get_stemming_probs(datafile, dataset)

    print test_sections
    
    print "* Build vocabulary..."
    sys.stdout.flush()
    
    # Build list of all destemmed words found in test sections
    destemmed_words = set()
    
    # Loop through all test sections
    for section in test_sections:
        
        # Loop through all word stems in each section
        for stemmed_section_word in stemmed_word_scores[section].keys():
            
            # Add all destemmed words for this section
            if not stem_probs.has_key( stemmed_section_word ):
                print "* Warning stem not found: " + stemmed_section_word
            else:
                destemmed_words.update(stem_probs[stemmed_section_word].keys())
            
    print "Total Unique Words: "+str(len(destemmed_words))

    srilm_temp_path = config._SRILM_LM_TEMP_FILE_PATH()
    f = open( srilm_temp_path, 'w')

    # First add all word - stopword and stopword - word pairs
    total_words = len(destemmed_words)
    for number, word in enumerate(destemmed_words):
        
        print "word : " + word + " Word " + str(number) + " of " + str(total_words)
        f.write(word+"\n")
    
    print "Done."

    # Run SRILM
    lmfile = config._SRILM_LM_FILE_PATH + "lm_arpa_" + dataset + "_" + str(order) + ".lmfile"
    lp_pipe = os.popen(config._SRILM_PATH + "ngram -lm " + lmfile + " -ppl " + srilm_temp_path + " -debug 3")
    lp_output = lp_pipe.read()
    lp_pipe.close()

    # Parse output
    print "Parsing output..."
    for lp_output_part in lp_output.split("\n\n"):
        lp_output_lines = lp_output_part.split("\n")
        
        if len(lp_output_lines[0].split(" ")) == 1 and len(lp_output_lines) > 2:
            # Get word
            word = lp_output_lines[0].strip()

            # Get start probability
            log_prob_parts = lp_output_lines[1].split(" ")
            if len(log_prob_parts) < 10:
                continue
            log_prob = log_prob_parts[9]
            try:
                start_word_probs[word] = math.e ** float(log_prob)
            except ValueError:
                continue

            # Get end probability
            log_prob_parts = lp_output_lines[2].split(" ")
            if len(log_prob_parts) < 10:
                continue
            log_prob = log_prob_parts[9]
            try:
                end_word_probs[word] = math.e ** float(log_prob)
            except ValueError:
                continue            

    return_tuple = (start_word_probs, end_word_probs)
    utils.save_pickle(pickle_path, return_tuple)
    return return_tuple

